#!/usr/bin/env python
#
# cyg-update.py
#
# Copyright (C) Koji Nakamaru, Dainichiro Kakei
# Modified: Aug 15 2008
# * commented out adjustMake().
# * added getSetupExe(). setup.exe is retrieved from cygwin.com.
# * added shutil.copyfile(setupini, setupini + '.bk') to backup setup.ini.
# * added only-newer option for lftp.exe.
# * removed setup.exe and make under cyg-update-lib.
# * changed the default ftp site to ftp.jaist.ac.jp.
# Modified: Jul 23 2007
# * added emacs-22 support.
# Modified: Feb 4 2007
# * added "cyg-update-lib" directory and stored files except this into it.
# * replaced modifysetupini() with adjustEmacs().
# * added adjustMake() to replace "make" with "make-3.80".
# * added -x "\\..*" to lftp options.
# Modified: Nov 26 2006
# * avoided to remove '\r' from setup.ini.
# Modified: Feb 7 2006
# * added code to check whether there is setup.ini.
# * made variables "mirrormode" and "noexec" local.
# Modified: Sep 17 2005
# * updated setup.exe to the cvs one and removed '-5' option for it.
# Modified: May 30 2005
# * added the most recent lftp.exe for supporting "ftp:list-empty-ok".
# Modified: May 27 2005
# * added the corrected setup.exe.
# Modified: May 26 2005
# * major refactoring by Kakei.
# Modified: May 20 2005
# * catch exception from os.remove().
# Modified: May 12 2005
# * remove setup.ini at first when mirroring.
# Created: May 08 2005
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import getopt
import os
import re
import shutil
import string
import sys


def usage():
    print """cyg-update.py

Usage: python cyg-update.py [options] [source]

Options:
  -m, --mirror            mirror mode on
  -n, --noexec            don't execute setup.exe
  -r ..., --remotedir=... use specified cygwin remote directory
  -t ..., --targetdir=... use specified local target directory(default:cygwin)
  -h, --help              show this help
"""

def main(argv):
    remotedir = 'ftp://ftp.jaist.ac.jp/pub/cygwin'
    targetdir = 'cygwin'
    mirrormode = False
    noexec = False
    try:
        opts, args = getopt.getopt(argv, "hmnr:t:", ["help", "mirror", "noexec", "remotedir=", "targetdir="])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-m", "--mirror"):
            mirrormode = True
        elif opt in ("-n", "--noexec"):
            noexec = True
        elif opt in ("-r", "--remotedir"):
            remotedir = arg
        elif opt in ("-t", "--target"):
            targetdir = arg
    setupini = os.path.join(targetdir, 'setup.ini')
    setupexe = os.path.join(targetdir, 'setup.exe')
    if mirrormode:
        try:
            os.remove(setupini)
        except:
            pass
        try:
            os.remove(setupini + '.bk')
        except:
            pass
        try:
            os.remove(setupexe)
        except:
            pass
        try:
            os.mkdir(targetdir)
        except:
            pass
        getBinaries(remotedir, targetdir)
        getSetupExe(setupexe)
        shutil.copyfile(setupini, setupini + '.bk')
        adjustEmacs(setupini, targetdir)
        #adjustMake(setupini, targetdir)
    if not os.access(setupini, os.R_OK):
        sys.stderr.write('\ncannot find setup.ini.\n')
    cygupdate(targetdir, noexec)

def getBinaries(remotedir, targetdir):
    """mirror by lftp"""
    ret = os.system(
        './cyg-update-lib/lftp -c \''
        + 'set ftp:list-empty-ok yes;'
        + 'mirror --delete --delete-first --only-newer '
        + '-x "^\\..*" '
        + '-x "setup.exe" '
        + '-x ".*-src.tar.bz2" '
        + '-x ".*-src.tar.gz" '
        + '-x mail-archives '
        + '-x release-2 '
        + '"' + remotedir + '" '
        + '"' + targetdir + '" '
        + '\'')
    if ret == 2:
        sys.stderr.write('\ninterrupted.\n')
        sys.exit(1)
    elif ret != 0:
        sys.stderr.write('\nfailed mirroring.\n')
        sys.exit(1)

def getSetupExe(setupexe):
    """get setup.exe from cygwin.com"""
    ret = os.system(
        './cyg-update-lib/lftp -c \''
        + 'set http:user-agent "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)";'
        + 'get '
        + '"http://www.cygwin.com/setup.exe" '
        + '-o '
        + '"' + setupexe + '" '
        + '\'')
    if ret == 2:
        sys.stderr.write('\ninterrupted.\n')
        sys.exit(1)
    elif ret != 0:
        sys.stderr.write('\nfailed to get setup.exe from cygwin.com.\n')
        sys.exit(1)
    #shutil.copyfile(os.path.join('cyg-update-lib', 'setup.exe'), setupexe)
    os.chmod(setupexe, 0755)

def adjustEmacs(setupini, targetdir):
    """adjust emacs"""
    re_s = re.compile('^@ emacs.*$')
    re_e = re.compile('^$')
    re_1 = re.compile('^version:')
    re_2 = re.compile('^version: (21\\.3\\.|22\\.).*')
    try:
        f = open(setupini, 'rb')
        lines = f.readlines()
        f.close()
    except:
        sys.stderr.write(
            'cannot open cygwin/setup.ini. please check "remotedir" in this script\n'
            'and your network configuration for mirroring cygwin.\n')
        sys.exit(1)
    os.chmod(setupini, 0644)
    f = open(setupini, 'wb')
    mode = 0
    for l in lines:
        l = l.replace('\n', '')
        if re_s.search(l):
            mode = 1
        if mode == 1 and re_1.search(l):
            mode = 2
        if mode != 0 and re_2.search(l):
            mode = 3
        if mode != 0 and re_e.search(l):
            mode = 0
        if mode == 0 or mode == 1 or mode == 3:
            f.write(l)
            f.write('\n')
    f.close()

def adjustMake(setupini, targetdir):
    """adjust make"""
    re_s = re.compile('^@ make.*$')
    re_e = re.compile('^$')
    re_1 = re.compile('^version:')
    try:
        f = open(setupini, 'rb')
        lines = f.readlines()
        f.close()
    except:
        sys.stderr.write(
            'cannot open cygwin/setup.ini. please check "remotedir" in this script\n'
            'and your network configuration for mirroring cygwin.\n')
        sys.exit(1)
    os.chmod(setupini, 0644)
    f = open(setupini, 'wb')
    mode = 0
    for l in lines:
        l = l.replace('\n', '')
        if re_s.search(l):
            mode = 1
        if mode == 1 and re_1.search(l):
            mode = 2
        if mode != 0 and re_e.search(l):
            f.write(
                'version: 3.80-1\n'
                'install: release/make/make-3.80-1.tar.bz2 286814 421acbc7a0605a117afb126e46d0f632\n'
                'source: release/make/make-3.80-1-src.tar.bz2 1260043 0b941f8c1003b4c90171c5f65b5501da\n');
            mode = 0
        if mode == 0 or mode == 1 or mode == 3:
            f.write(l)
            f.write('\n')
    f.close()
    src = os.path.join('cyg-update-lib', 'make')
    dst = os.path.join(os.path.join(targetdir, 'release'), 'make')
    shutil.rmtree(dst)
    shutil.copytree(src, dst)

def cygupdate(targetdir, noexec):
    """execute setup.exe"""
    try:
        os.chdir(targetdir)
    except:
        sys.stderr.write(
            'cannot chdir cygwin. please mirror it by specifying "--mirror" option\n'
            'with an adequate setting of "remotedir" in this script.\n');
        sys.exit(1)

    cwd = ''
    try:
        f = os.popen('cygpath -ams .', 'rb')
        cwd = f.readline()
        cwd = cwd.replace('\r', '')
        cwd = cwd.replace('\n', '')
        f.close()
    except:
        sys.stderr.write(
            'cannot invoke cygpath correctly. please check your cygwin installation.\n')
        sys.exit(1)
    if not noexec:
        os.execv('./setup.exe', ['./setup.exe', '-L', '-l', cwd, '-q'])


if __name__ == "__main__":
    main(sys.argv[1:])
